package org.jeecg.modules.system.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Setter;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.exception.JeecgBootException;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.system.app.dto.AppGameDTO;
import org.jeecg.modules.system.app.entity.AppGame;
import org.jeecg.modules.system.app.entity.AppGamePriceRules;
import org.jeecg.modules.system.app.mapper.AppGameMapper;
import org.jeecg.modules.system.app.mapper.AppGamePriceRulesMapper;
import org.jeecg.modules.system.app.service.IAppGameService;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

import static org.jeecg.common.constant.CommonConstant.SC_INTERNAL_SERVER_ERROR_500;

/**
 * @Description: 赛事表
 * @Author: jeecg-boot
 * @Date:   2025-07-03
 * @Version: V1.0
 */
@Service
public class AppGameServiceImpl extends ServiceImpl<AppGameMapper, AppGame> implements IAppGameService {
    @Resource
    private AppGamePriceRulesMapper rulesMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean saveWitchPriceRules(AppGameDTO gameDTO) {
        AppGame game=gameDTO.getGame();
        if (null==game.getType()) throw new JeecgBootException("请选择赛事类型");
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        game.setOrgCode(sysUser.getOrgCode());
        game.setTenantId(sysUser.getOrgId());
        int saveGameResult = baseMapper.insert(game);
        if (saveGameResult<1) {
            throw new JeecgBootException("赛事信息保存失败");
        }
        List<AppGamePriceRules> priceRulesList = gameDTO.getGamePriceRules();
        for (AppGamePriceRules rule : priceRulesList) {
            rule.setGameId(game.getId());
            rule.setOrgCode(game.getOrgCode());
            rule.setTenantId(game.getTenantId());
            int savePriceResult = rulesMapper.insert(rule); // 保证每个操作都在事务中
            if (savePriceResult<1) {
                throw new JeecgBootException("赛事价格规则保存失败");
            }
        }
        return Boolean.TRUE;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean editWitchPriceRules(AppGameDTO gameDTO) {
        AppGame game=gameDTO.getGame();
        if (null==game.getType()) throw new JeecgBootException("请选择赛事类型");
        AppGame dbGame = baseMapper.selectById(game.getId());
        if (null==dbGame) throw new JeecgBootException("未找到对应数据", SC_INTERNAL_SERVER_ERROR_500);
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        checkPermission(sysUser,dbGame);
        game.setOrgCode(sysUser.getOrgCode());
        game.setTenantId(sysUser.getOrgId());
        int updateGameResult = baseMapper.updateById(game);
        if (updateGameResult<1) {
            throw new JeecgBootException("赛事信息保存失败");
        }
        List<AppGamePriceRules> priceRulesList = gameDTO.getGamePriceRules();
        for (AppGamePriceRules rule : priceRulesList) {
            rule.setGameId(game.getId());
            rule.setOrgCode(game.getOrgCode());
            rule.setTenantId(game.getTenantId());
            int savePriceResult = rulesMapper.updateById(rule);
            if (savePriceResult<1) {
                throw new JeecgBootException("赛事价格规则保存失败");
            }
        }
        return Boolean.TRUE;
    }

    @Override
    public AppGameDTO queryWitchPriceRulesById(String id) {
        AppGame dbGame = baseMapper.selectById(id);
        if (null==dbGame) throw new JeecgBootException("未找到对应数据", SC_INTERNAL_SERVER_ERROR_500);
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        checkPermission(sysUser,dbGame);
        List<AppGamePriceRules> priceRulesList = rulesMapper.selectList(Wrappers.<AppGamePriceRules>lambdaQuery().eq(AppGamePriceRules::getGameId, id));
        return new AppGameDTO(dbGame,priceRulesList);
    }

    @Override
    public Boolean deleteWitchPriceRulesById(String id) {
        AppGame dbGame = baseMapper.selectById(id);
        if (null==dbGame) throw new JeecgBootException("未找到对应数据", SC_INTERNAL_SERVER_ERROR_500);
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        checkPermission(sysUser,dbGame);
        int deleteGameResult = baseMapper.deleteById(id);
        if (deleteGameResult<1) {
            throw new JeecgBootException("赛事信息删除失败");
        }
        return Boolean.TRUE;
    }

    /**
     *  校验权限
     * @param sysUser
     * @param game
     */
    private void checkPermission(LoginUser sysUser, AppGame game) {
        if (!sysUser.getOrgCode().equals(game.getOrgCode())) throw new JeecgBootException("无权限操作", SC_INTERNAL_SERVER_ERROR_500);
    }
}
